export interface JsonCodecPatch {
    id: JsonCodecTimestamp;
    ops: JsonCodecOperation[];
    meta?: unknown;
}
export type JsonCodecOperation = JsonCodecNewConOperation | JsonCodecNewValOperation | JsonCodecNewObjOperation | JsonCodecNewVecOperation | JsonCodecNewStrOperation | JsonCodecNewBinOperation | JsonCodecNewArrOperation | JsonCodecInsValOperation | JsonCodecInsObjOperation | JsonCodecInsVecOperation | JsonCodecInsStrOperation | JsonCodecInsBinOperation | JsonCodecInsArrOperation | JsonCodecDelOperation | JsonCodecNopOperation;
export type JsonCodecTimestamp = [
    sessionId: number,
    time: number
] | number;
export type JsonCodecTimespan = [
    sessionId: number,
    time: number,
    span: number
];
export interface JsonCodecOperationBase<T> {
    op: T;
}
export interface JsonCodecNewConOperation extends JsonCodecOperationBase<'con'> {
    value: unknown | JsonCodecTimestamp;
    timestamp?: boolean;
}
export interface JsonCodecNewValOperation extends JsonCodecOperationBase<'val'> {
    value: JsonCodecTimestamp;
}
export type JsonCodecNewObjOperation = JsonCodecOperationBase<'obj'>;
export type JsonCodecNewVecOperation = JsonCodecOperationBase<'vec'>;
export type JsonCodecNewStrOperation = JsonCodecOperationBase<'str'>;
export type JsonCodecNewBinOperation = JsonCodecOperationBase<'bin'>;
export type JsonCodecNewArrOperation = JsonCodecOperationBase<'arr'>;
export interface JsonCodecObjectOperation<T> extends JsonCodecOperationBase<T> {
    obj: JsonCodecTimestamp;
}
export interface JsonCodecInsValOperation extends JsonCodecObjectOperation<'ins_val'> {
    value: JsonCodecTimestamp;
}
export interface JsonCodecInsObjOperation extends JsonCodecObjectOperation<'ins_obj'> {
    value: Array<[key: string, value: JsonCodecTimestamp]>;
}
export interface JsonCodecInsVecOperation extends JsonCodecObjectOperation<'ins_vec'> {
    value: Array<[index: number, value: JsonCodecTimestamp]>;
}
export interface JsonCodecInsStrOperation extends JsonCodecObjectOperation<'ins_str'> {
    after?: JsonCodecTimestamp;
    value: string;
}
export interface JsonCodecInsBinOperation extends JsonCodecObjectOperation<'ins_bin'> {
    after?: JsonCodecTimestamp;
    value: string;
}
export interface JsonCodecInsArrOperation extends JsonCodecObjectOperation<'ins_arr'> {
    after?: JsonCodecTimestamp;
    values: JsonCodecTimestamp[];
}
export interface JsonCodecDelOperation extends JsonCodecObjectOperation<'del'> {
    what: JsonCodecTimespan[];
}
export interface JsonCodecNopOperation extends JsonCodecOperationBase<'nop'> {
    len?: number;
}
