"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_1 = require("fs");
var msgpack_1 = require("../json-pack/msgpack");
var CborDecoder_1 = require("../json-pack/cbor/CborDecoder");
var JSONB = tslib_1.__importStar(require("../json-binary"));
var arg_1 = tslib_1.__importDefault(require("arg"));
var args = (0, arg_1.default)({
    '--format': String,
    '--cbor': Boolean,
}, {
    argv: process.argv,
});
var ALLOWED_FORMATS = new Set(['msgpack', 'messagepack', 'cbor']);
var format = args['--cbor']
    ? 'cbor'
    : String((_a = args['--format']) !== null && _a !== void 0 ? _a : 'msgpack').toLowerCase();
if (!ALLOWED_FORMATS.has(format))
    throw new Error("Unknown format: ".concat(format));
try {
    switch (format) {
        case 'msgpack':
        case 'messagepack': {
            var decoder = new msgpack_1.MsgPackDecoderFast();
            var buf = (0, fs_1.readFileSync)(0);
            var arr = new Uint8Array(buf.length);
            for (var i = 0; i < buf.length; i++)
                arr[i] = buf[i];
            var decoded = decoder.decode(arr);
            var json = JSONB.stringify(decoded);
            process.stdout.write(json);
            break;
        }
        case 'cbor': {
            var decoder = new CborDecoder_1.CborDecoder();
            var buf = (0, fs_1.readFileSync)(0);
            var arr = new Uint8Array(buf.length);
            for (var i = 0; i < buf.length; i++)
                arr[i] = buf[i];
            var decoded = decoder.decode(arr);
            var json = JSONB.stringify(decoded);
            process.stdout.write(json);
            break;
        }
    }
}
catch (error) {
    var output = error instanceof Error ? error.message : String(error);
    process.stderr.write(output + '\n');
    process.exit(1);
}
