"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UbjsonDecoder = void 0;
var Reader_1 = require("../../util/buffers/Reader");
var JsonPackExtension_1 = require("../JsonPackExtension");
var UbjsonDecoder = (function () {
    function UbjsonDecoder() {
        this.reader = new Reader_1.Reader();
    }
    UbjsonDecoder.prototype.read = function (uint8) {
        this.reader.reset(uint8);
        return this.readAny();
    };
    UbjsonDecoder.prototype.readAny = function () {
        var reader = this.reader;
        var octet = reader.u8();
        switch (octet) {
            case 0x5a:
                return null;
            case 0x54:
                return true;
            case 0x46:
                return false;
            case 0x55:
                return reader.u8();
            case 0x69:
                return reader.i8();
            case 0x49: {
                var int = reader.view.getInt16(reader.x, false);
                reader.x += 2;
                return int;
            }
            case 0x6c: {
                var int = reader.view.getInt32(reader.x, false);
                reader.x += 4;
                return int;
            }
            case 0x64: {
                var num = reader.view.getFloat32(reader.x, false);
                reader.x += 4;
                return num;
            }
            case 0x44: {
                var num = reader.view.getFloat64(reader.x, false);
                reader.x += 8;
                return num;
            }
            case 0x4c: {
                var num = reader.view.getBigInt64(reader.x, false);
                reader.x += 8;
                return num;
            }
            case 0x53:
                return reader.utf8(+this.readAny());
            case 0x43:
                return String.fromCharCode(reader.u8());
            case 0x5b: {
                var uint8 = reader.uint8;
                var x = reader.x;
                if (uint8[x] === 0x24 && uint8[x + 1] === 0x55 && uint8[x + 2] === 0x23) {
                    reader.x += 3;
                    var size = +this.readAny();
                    return reader.buf(size);
                }
                var type = -1;
                if (uint8[x] === 0x24) {
                    reader.x++;
                    type = reader.u8();
                }
                var count = -1;
                if (uint8[x] === 0x23) {
                    reader.x++;
                    count = reader.u8();
                }
                if (uint8[x] === 0x24) {
                    reader.x++;
                    type = reader.u8();
                }
                if (count >= 0) {
                    var wordSize = 1;
                    switch (type) {
                        case 0x49:
                            wordSize = 2;
                            break;
                        case 0x6c:
                        case 0x64:
                            wordSize = 4;
                            break;
                        case 0x44:
                        case 0x4c:
                            wordSize = 8;
                            break;
                    }
                    return new JsonPackExtension_1.JsonPackExtension(type, reader.buf(count * wordSize));
                }
                else {
                    var arr = [];
                    while (uint8[reader.x] !== 0x5d)
                        arr.push(this.readAny());
                    reader.x++;
                    return arr;
                }
            }
            case 0x7b: {
                var uint8 = reader.uint8;
                var obj = {};
                while (uint8[reader.x] !== 0x7d) {
                    var keySize = +this.readAny();
                    var key = reader.utf8(keySize);
                    obj[key] = this.readAny();
                }
                reader.x++;
                return obj;
            }
            case 0x4e:
                return undefined;
        }
        return;
    };
    return UbjsonDecoder;
}());
exports.UbjsonDecoder = UbjsonDecoder;
