"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFromBase64 = void 0;
var constants_1 = require("./constants");
var E = '=';
var createFromBase64 = function (chars) {
    if (chars === void 0) { chars = constants_1.alphabet; }
    if (chars.length !== 64)
        throw new Error('chars must be 64 characters long');
    var max = 0;
    for (var i = 0; i < chars.length; i++)
        max = Math.max(max, chars.charCodeAt(i));
    var table = [];
    for (var i = 0; i <= max; i += 1)
        table[i] = -1;
    for (var i = 0; i < chars.length; i++)
        table[chars.charCodeAt(i)] = i;
    return function (encoded) {
        if (!encoded)
            return new Uint8Array(0);
        var length = encoded.length;
        if (length % 4 !== 0)
            throw new Error('Base64 string length must be a multiple of 4');
        var mainLength = encoded[length - 1] !== E ? length : length - 4;
        var bufferLength = (length >> 2) * 3;
        var padding = 0;
        if (encoded[length - 2] === E) {
            padding = 2;
            bufferLength -= 2;
        }
        else if (encoded[length - 1] === E) {
            padding = 1;
            bufferLength -= 1;
        }
        var buf = new Uint8Array(bufferLength);
        var j = 0;
        var i = 0;
        for (; i < mainLength; i += 4) {
            var sextet0 = table[encoded.charCodeAt(i)];
            var sextet1 = table[encoded.charCodeAt(i + 1)];
            var sextet2 = table[encoded.charCodeAt(i + 2)];
            var sextet3 = table[encoded.charCodeAt(i + 3)];
            if (sextet0 < 0 || sextet1 < 0 || sextet2 < 0 || sextet3 < 0)
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
            buf[j + 1] = (sextet1 << 4) | (sextet2 >> 2);
            buf[j + 2] = (sextet2 << 6) | sextet3;
            j += 3;
        }
        if (padding === 2) {
            var sextet0 = table[encoded.charCodeAt(mainLength)];
            var sextet1 = table[encoded.charCodeAt(mainLength + 1)];
            if (sextet0 < 0 || sextet1 < 0)
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
        }
        else if (padding === 1) {
            var sextet0 = table[encoded.charCodeAt(mainLength)];
            var sextet1 = table[encoded.charCodeAt(mainLength + 1)];
            var sextet2 = table[encoded.charCodeAt(mainLength + 2)];
            if (sextet0 < 0 || sextet1 < 0 || sextet2 < 0)
                throw new Error('INVALID_BASE64_STRING');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
            buf[j + 1] = (sextet1 << 4) | (sextet2 >> 2);
        }
        return buf;
    };
};
exports.createFromBase64 = createFromBase64;
