"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFromBase64Bin = void 0;
var constants_1 = require("./constants");
var createFromBase64Bin = function (chars, paddingOctet) {
    if (chars === void 0) { chars = constants_1.alphabet; }
    if (paddingOctet === void 0) { paddingOctet = 0x3d; }
    if (chars.length !== 64)
        throw new Error('chars must be 64 characters long');
    var max = 0;
    for (var i = 0; i < chars.length; i++)
        max = Math.max(max, chars.charCodeAt(i));
    var table = [];
    for (var i = 0; i <= max; i += 1)
        table[i] = -1;
    for (var i = 0; i < chars.length; i++)
        table[chars.charCodeAt(i)] = i;
    return function (view, offset, length) {
        if (!length)
            return new Uint8Array(0);
        if (length % 4 !== 0)
            throw new Error('Base64 string length must be a multiple of 4');
        var end = offset + length;
        var last = end - 1;
        var lastOctet = view.getUint8(last);
        var mainEnd = offset + (lastOctet !== paddingOctet ? length : length - 4);
        var bufferLength = (length >> 2) * 3;
        var padding = 0;
        if (last > 0 && view.getUint8(last - 1) === paddingOctet) {
            padding = 2;
            bufferLength -= 2;
        }
        else if (lastOctet === paddingOctet) {
            padding = 1;
            bufferLength -= 1;
        }
        var buf = new Uint8Array(bufferLength);
        var j = 0;
        var i = offset;
        for (; i < mainEnd; i += 4) {
            var word = view.getUint32(i);
            var octet0 = word >>> 24;
            var octet1 = (word >>> 16) & 0xff;
            var octet2 = (word >>> 8) & 0xff;
            var octet3 = word & 0xff;
            var sextet0 = table[octet0];
            var sextet1 = table[octet1];
            var sextet2 = table[octet2];
            var sextet3 = table[octet3];
            if (sextet0 < 0 || sextet1 < 0 || sextet2 < 0 || sextet3 < 0)
                throw new Error('INVALID_BASE64_SEQ');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
            buf[j + 1] = (sextet1 << 4) | (sextet2 >> 2);
            buf[j + 2] = (sextet2 << 6) | sextet3;
            j += 3;
        }
        if (padding === 2) {
            var word = view.getUint16(mainEnd);
            var octet0 = word >> 8;
            var octet1 = word & 0xff;
            var sextet0 = table[octet0];
            var sextet1 = table[octet1];
            if (sextet0 < 0 || sextet1 < 0)
                throw new Error('INVALID_BASE64_SEQ');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
        }
        else if (padding === 1) {
            var word = view.getUint16(mainEnd);
            var octet0 = word >> 8;
            var octet1 = word & 0xff;
            var octet2 = view.getUint8(mainEnd + 2);
            var sextet0 = table[octet0];
            var sextet1 = table[octet1];
            var sextet2 = table[octet2];
            if (sextet0 < 0 || sextet1 < 0 || sextet2 < 0)
                throw new Error('INVALID_BASE64_SEQ');
            buf[j] = (sextet0 << 2) | (sextet1 >> 4);
            buf[j + 1] = (sextet1 << 4) | (sextet2 >> 2);
        }
        return buf;
    };
};
exports.createFromBase64Bin = createFromBase64Bin;
